import os
import platform
import subprocess
import wave
import speech_recognition as sr
from gtts import gTTS
import pygame
import sounddevice as sd
import webbrowser
import pyaudio
import json
from settings import *

def play_audio(aimp_path, mp3_file_path):
    if os.path.exists(mp3_file_path):
        # قم بتشغيل AIMP وتمرير الملف الصوتي إليه
        subprocess.Popen([aimp_path, mp3_file_path])
        return "opened"
    else:
        return "Not Found File"

def stop_recording():
    p = pyaudio.PyAudio()
    for i in range(p.get_device_count()):
        p.terminate()

def delete_all_files(directory=".", type={}):
    for filename in os.listdir(directory):
        for index, type in type.items():
            if filename.endswith((type)):
                filepath = os.path.join(directory, filename)
                try:
                    os.remove(filepath)
                    return f"Deleted: {filepath}"
                except OSError as e:
                    print (f"Error deleting {filepath}: {e}")
                    return f"Error deleting {filepath}: {e}"

def create_file(name):

    print("Please enter the text or code (press Ctrl + D on Unix or Ctrl + Z then Enter on Windows to finish):")

    user_input_lines = []
    try:
        while True:
            line = input()
            user_input_lines.append(line)
    except EOFError:
        pass

    # Merge the entered lines into a single text
    user_input = '\n'.join(user_input_lines)

    # Write the entered text to the file
    filename = f"{name}.py"

    # Write the entered text to the file using Unicode encoding
    with open(filename, "w", encoding="utf-8") as file:
        file.write(user_input)

    return "created"

def open_website(url):
    try:
        webbrowser.open(url)
        return "opened"
    except Exception as e:
        print ("An error occurred:", e)
        return "An error occurred:", e

def open_file(filepath):
    try:
        if os.path.exists(filepath):
            subprocess.Popen([str(filepath)])
            return "open"
        else:
            return "Not Found Path"
    except Exception as e:
       print ("An error occurred:", e)
       return "An error occurred:", e

def shutdown():
    system = platform.system()
    if system == "Windows":
        subprocess.run(["shutdown", "/s", "/t", "1"])
    elif system == "Linux" or system == "Darwin":  # Darwin is the system name for macOS
        subprocess.run(["sudo", "shutdown", "-h", "now"])
    else:
        raise NotImplementedError("Unsupported OS")

def record_audio(filename="recorder.wav", duration=5, fs=44100):
    sd.default.device = 2
    try:
        audio_data = sd.rec(int(duration * fs), samplerate=fs, channels=1, dtype='int16')
        sd.wait()
        with wave.open(filename, 'wb') as wf:
            wf.setnchannels(1)
            wf.setsampwidth(2)
            wf.setframerate(fs)
            wf.writeframes(audio_data.tobytes())
        return "saved"
    except Exception as e:
        print ("An error occurred:", e)
        return "An error occurred:", e

def transcribe_audio(filename="recorder.wav"):
    recognizer = sr.Recognizer()
    with sr.AudioFile(filename) as source:
        audio = recognizer.record(source)
        try:
            query = recognizer.recognize_google(audio, language='en-US')
            return query
        except sr.UnknownValueError:
            return ""
        except sr.RequestError as e:
            print (f"Could not request results; {e}")
            return f"Could not request results; {e}"

def text_to_speech(text, filename="tts.mp3"):
    tts = gTTS(text, lang='en')
    tts.save(filename)
    return "saved"

def play_sound(filename="tts.mp3"):
    pygame.mixer.init()
    sound = pygame.mixer.Sound(filename)
    sound.play()
    while pygame.mixer.get_busy():
        pygame.time.Clock().tick(10)
    sound.stop()
    return "played"

def restart():
    system = platform.system()
    if system == "Windows":
        os.system("shutdown /r /t 1")

def log_off():
    system = platform.system()
    if system == "Windows":
        os.system("shutdown /l")

def hibernate():
    system = platform.system()
    if system == "Windows":
        os.system("shutdown /h")

def play_audio_online(aimp_path, mp3_file_link):
        # قم بتشغيل AIMP وتمرير الملف الصوتي إليه
        subprocess.Popen([aimp_path, mp3_file_link])
        return "opened"

def token_information(data, type='binance'):

    if type == 'binance':
        link = "https://bscscan.com/token/" + str(data)
        open_website(link)
        return "opened"
    elif type == 'etherum':
        link = "https://etherscan.io/token/" + str(data)
        open_website(link)
        return "opened"
    elif type == 'geckoterminal':
        link = 'https://ywp.freewebhostmost.com/really/token.php?pool=' + str(data)
        return "opened"
    else:
        return "UnSupported type"
    
def add_task(task_name, task_dis):
    tasks[task_name] = task_dis
    marks[task_name] = "no"
    with open(tasks_file, "w") as f:
        json.dump(tasks, f)
    with open(marks_file, "w") as f:
        json.dump(marks, f)
    return 'added'

def delete_all_tasks():
    tasks = {}
    marks = {}

    with open (tasks_file, "w") as file:
        json.dump(tasks, file)
    
    with open (marks_file, "w") as file:
        json.dump(marks, file)

    return 'deleted'

def delete_task(task_name):
    if task_name in tasks:
        del tasks[task_name]
        del marks[task_name]
        with open(tasks_file, "w") as f:
            json.dump(tasks, f)
        with open(marks_file, "w") as f:
            json.dump(marks, f)
        return 'deleted'
    else:
        return 'not found'
    
def edit_mark_done_task(task_name, new_mark="yes"):
    if new_mark == "yes" or new_mark == "no":
        a = 0
    else:
        return 'not marked'
    if task_name in tasks:
        marks[task_name] = new_mark
        with open(marks_file, "w") as f:
            json.dump(marks, f)
        return 'marked'
    else:
        return 'not found'
    
def edit_tasks(task_name, new_task_dis):
    if task_name in tasks:
        tasks[task_name] = new_task_dis
        with open(tasks_file, "w") as f:
            json.dump(tasks, f)
        return 'edited'
    else:
        return 'not_found'
    
def load_tasks():

    with open (tasks_file, "w") as file:
        tasks = json.load(file)
    
    with open (marks_file, "w") as file:
        marks = json.load(file)

    return 'loaded'

def tasks():
    if os.path.exists(tasks_file):
        with open(tasks_file, "r") as f:
            tasks = json.load(f)
    else:
        tasks = {}
    return tasks

def marks():
    if os.path.exists(marks_file):
        with open(marks_file, "r") as f:
            marks = json.load(f)
    else:
        marks = {}
    return marks

def tasks_file():
    return tasks_file

def marks_file():
    return marks_file


import os
import platform

def restart():
    system = platform.system()
    if system == "Windows":
        os.system("shutdown /r /t 1")
    else:
        raise NotImplementedError("Unsupported OS")


import os
import json

setting_file = "settings.json"
tasks_file = "tasks.json"
marks_file = "marks.json"
tasks = {}
marks = {}

if os.path.exists(tasks_file):
    with open(tasks_file, "r") as f:
        tasks = json.load(f)

if os.path.exists(marks_file):
    with open(marks_file, "r") as f:
        marks = json.load(f)

def tasks():
    if os.path.exists(tasks_file):
        with open(tasks_file, "r") as f:
            tasks = json.load(f)
    else:
        tasks = {}
    return tasks

def marks():
    if os.path.exists(marks_file):
        with open(marks_file, "r") as f:
            marks = json.load(f)
    else:
        marks = {}
    return marks

def tasks_file():
    return tasks_file

def marks_file():
    return marks_file



import json
import settings

def edit_task(task_name, new_task_dis):
    tasks = settings.tasks()
    marks = settings.marks()
    tasks_file = settings.tasks_file()
    marks_file = settings.marks_file()

    if task_name in tasks:
        tasks[task_name] = new_task_dis
        with open(tasks_file, "w") as f:
            json.dump(tasks, f)

        settings.tasks = tasks
        settings.marks = marks

        return 'edited'
    else:
        return 'not_found'


import json
import settings

def delete_all_tasks():
    tasks = {}
    marks = {}
    tasks_file = settings.tasks_file()
    marks_file = settings.marks_file()

    with open (tasks_file, "w") as file:
        json.dump(tasks, file)
    
    with open (marks_file, "w") as file:
        json.dump(marks, file)

    settings.tasks = tasks
    settings.marks = marks

    return 'deleted'


from .open_website import open_website

def token_information(data, type='binance'):

    if type == 'binance':
        link = "https://bscscan.com/token/" + str(data)
        open_website(link)
        return "opened"
    elif type == 'etherum':
        link = "https://etherscan.io/token/" + str(data)
        open_website(link)
        return "opened"
    elif type == 'geckoterminal':
        link = 'https://ywp.freewebhostmost.com/really/token.php?pool=' + str(data)
        return "opened"
    else:
        return "UnSupported type"


import os
import subprocess

def open_file(filepath):
    try:
        if os.path.exists(filepath):
            subprocess.Popen([str(filepath)])
            return "open"
        else:
            return "Not Found Path"
    except Exception as e:
       print ("An error occurred:", e)
       return "An error occurred:", e


import json
import settings

def load_tasks():
    tasks_file = settings.tasks_file()
    marks_file = settings.marks_file()

    with open (tasks_file, "w") as file:
        tasks = json.load(file)
    
    with open (marks_file, "w") as file:
        marks = json.load(file)

    settings.tasks = tasks
    settings.marks = marks

    return 'loaded'


import os

def delete_all_files(directory=".", type={}):
    for filename in os.listdir(directory):
        for index, type in type.items():
            if filename.endswith((type)):
                filepath = os.path.join(directory, filename)
                try:
                    os.remove(filepath)
                    return f"Deleted: {filepath}"
                except OSError as e:
                    print (f"Error deleting {filepath}: {e}")
                    return f"Error deleting {filepath}: {e}"


import wave
import sounddevice as sd

def record_audio(filename="recorder.wav", duration=5, fs=44100):
    sd.default.device = 2
    try:
        audio_data = sd.rec(int(duration * fs), samplerate=fs, channels=1, dtype='int16')
        sd.wait()
        with wave.open(filename, 'wb') as wf:
            wf.setnchannels(1)
            wf.setsampwidth(2)
            wf.setframerate(fs)
            wf.writeframes(audio_data.tobytes())
        return "saved"
    except Exception as e:
        print ("An error occurred:", e)
        return "An error occurred:", e


import json
import settings

def add_task(task_name, task_dis):
    tasks = settings.tasks()
    marks = settings.marks()
    tasks_file = settings.tasks_file()
    marks_file = settings.marks_file()

    tasks[task_name] = task_dis
    marks[task_name] = "no"
    with open(tasks_file, "w") as f:
        json.dump(tasks, f)
    with open(marks_file, "w") as f:
        json.dump(marks, f)

    settings.tasks = tasks
    settings.marks = marks

    return 'added'


import pygame

def play_sound(filename="tts.mp3"):
    pygame.mixer.init()
    sound = pygame.mixer.Sound(filename)
    sound.play()
    while pygame.mixer.get_busy():
        pygame.time.Clock().tick(10)
    sound.stop()
    return "played"


import json
import settings

def task_mark_done(task_name, new_mark="yes"):
    tasks = settings.tasks()
    marks = settings.marks()
    tasks_file = settings.tasks_file()
    marks_file = settings.marks_file()

    if new_mark == "yes" or new_mark == "no":
        a = 0
    else:
        return 'not marked'
    if task_name in tasks:
        marks[task_name] = new_mark
        with open(marks_file, "w") as f:
            json.dump(marks, f)

        settings.tasks = tasks
        settings.marks = marks
        
        return 'marked'
    else:
        return 'not found'


import speech_recognition as sr

def transcribe_audio(filename="recorder.wav"):
    recognizer = sr.Recognizer()
    with sr.AudioFile(filename) as source:
        audio = recognizer.record(source)
        try:
            query = recognizer.recognize_google(audio, language='en-US')
            return query
        except sr.UnknownValueError:
            return ""
        except sr.RequestError as e:
            print (f"Could not request results; {e}")
            return f"Could not request results; {e}"


import webbrowser

def open_website(url):
    try:
        webbrowser.open(url)
        return "opened"
    except Exception as e:
        print ("An error occurred:", e)
        return "An error occurred:", e


import os
import subprocess

def play_audio(pro_path, mp3_file_path):
    if os.path.exists(mp3_file_path):
        # قم بتشغيل AIMP وتمرير الملف الصوتي إليه
        subprocess.Popen([pro_path, mp3_file_path])
        return "opened"
    else:
        return "Not Found File"


# import os
from setuptools import setup, find_packages
# from setuptools.command.install import install as InstallCommand

# class CustomInstallCommand(InstallCommand):
#     def run(self):
#         # Download the required file from a URL
#         url = 'https://example.com/aimp.exe'
#         os.system(f'curl -o aimp.exe {url}')  # Use appropriate command for downloading

#         # Continue with the regular installation process
#         super().run()

setup(
    name="YWP",
    version="1.1.0",
    packages=find_packages(),
    install_requires=[
        "SpeechRecognition",
        "gtts",
        "pygame",
        "sounddevice",
        "pyaudio"
    ],
    # package_data={
    #     '': ['aimp.exe'],  # Include aimp.exe at the root level of the package
    # },
    # data_files=[('', ['aimp.exe'])],  # Another way to include aimp.exe
    # entry_points={
    #     'console_scripts': [
    #         'YWP_install_AIMP = YWP:AIMP_Run', 
    #     ],
    # },
    author="Your Wanted Products",
    author_email="pbstzidr@ywp.freewebhostmost.com",
    description="A big Package has a lot of things",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    # url="https://github.com/YourWantedProducts/YWP_Python",
    # cmdclass={
    #     'install': CustomInstallCommand,
    # },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


from .play_audio import play_audio
from .stop_recording import stop_recording
from .create_file import create_file
from .open_file import open_file
from .open_website import open_website
from .shutdown import shutdown
from .record_audio import record_audio
from .transcribe_audio import transcribe_audio
from .text_to_speech import text_to_speech
from .play_sound import play_sound
from .restart import restart
from .log_off import log_off
from .hibernate import hibernate
from .play_audio_online import play_audio_online
from .token_information import token_information
from .load_tasks import load_tasks
from .delete_all_tasks import delete_all_tasks
from .add_task import add_task
from .edit_mark_done import task_mark_done
from .edit_task import edit_task
from .delete_task import delete_task
from .settings import tasks
from .settings import marks
from .settings import tasks_file
from .settings import marks_file



import platform
import subprocess

def shutdown():
    system = platform.system()
    if system == "Windows":
        subprocess.run(["shutdown", "/s", "/t", "1"])
    elif system == "Linux" or system == "Darwin":  # Darwin is the system name for macOS
        subprocess.run(["sudo", "shutdown", "-h", "now"])
    else:
        raise NotImplementedError("Unsupported OS")


def create_file(name):

    print("Please enter the text or code (press Ctrl + D on Unix or Ctrl + Z then Enter on Windows to finish):")

    user_input_lines = []
    try:
        while True:
            line = input()
            user_input_lines.append(line)
    except EOFError:
        pass

    # Merge the entered lines into a single text
    user_input = '\n'.join(user_input_lines)

    # Write the entered text to the file
    filename = f"{name}.py"

    # Write the entered text to the file using Unicode encoding
    with open(filename, "w", encoding="utf-8") as file:
        file.write(user_input)

    return "created"